// import { login, logClear, getInfo } from '@/api/user'
import { user } from '@/api'
import {
  getToken,
  setToken,
  removeToken,
  getUserInfo,
  setUserInfo,
  removeUserInfo,
} from '@/utils/auth'
import { resetRouter } from '@/router'

const getDefaultState = () => {
  return {
    token: getToken(),
    userInfo: getUserInfo(),
    name: '',
    avatar: '',
    roles: [],
    menus: [],
    authButtons: [],
  }
}

const state = getDefaultState()

const mutations = {
  RESET_STATE: (state) => {
    Object.assign(state, getDefaultState())
  },
  SET_TOKEN: (state, token) => {
    state.token = token
  },
  SET_USERINFO: (state, userInfo) => {
    state.userInfo = userInfo
  },
  SET_NAME: (state, name) => {
    state.name = name
  },
  SET_AVATAR: (state, avatar) => {
    state.avatar = avatar
  },
  SET_ROLES: (state, roles) => {
    state.roles = roles // 角色权限
  },
  SET_MENUS: (state, menus) => {
    state.menus = menus // 菜单权限
  },
  SET_AUTHBUTTONS: (state, authButtons) => {
    state.authButtons = authButtons // 按钮权限
  },
}

const actions = {
  // user login
  login({ commit, dispatch }, userInfo) {
    const { username, password } = userInfo
    return new Promise((resolve, reject) => {
      user
        .login({ username: username.trim(), password })
        .then((response) => {
          const { result } = response
          commit('SET_TOKEN', result.token)
          commit('SET_USERINFO', result)
          // localStorage.setItem('name',result.username)
          setToken(result.token)
          setUserInfo(result)
          // dispatch("getAuthButtons")
          // auth.getAuthDetail(result.id).then((res) => {
          //   //response.result.role 用户角色固有权限
          //   const roleIdArr = res.result.role.auth.map((i) => {
          //     return i.id;
          //   });
          //   //response.result.auth 用户已有权限
          //   const authIdArr = res.result.auth.map((i) => {
          //     return i.id;
          //   });

          //   const authButtons = [...roleIdArr,...authIdArr]
          //   commit('SET_AUTHBUTTONS', authButtons)
          // });

          resolve()
        })
        .catch((error) => {
          reject(error)
        })
    })
  },

  // 获取button权限
  // getAuthButtons({ commit, state }) {
  //   return new Promise((resolve, reject) => {
  //     const id = JSON.parse(getUserInfo()).id
  //     auth.getAuthDetail(id).then((response) => {
  //       let roleIdArr = [],
  //         authIdArr = []
  //       // response.result.role 用户角色固有权限
  //       if (response.result.role) {
  //         roleIdArr = response.result.role.auth.map((i) => {
  //           return i.id
  //         })
  //       }
  //       // response.result.auth 用户已有权限
  //       if (response.result.auth) {
  //         authIdArr = response.result.auth.map((i) => {
  //           return i.id
  //         })
  //       }

  //       const authButtons = [...roleIdArr, ...authIdArr]
  //       commit('SET_AUTHBUTTONS', authButtons)
  //       resolve()
  //     })
  //   })
  // },

  // get user info
  /* 获取异步路由后加入404路由能解决刷新后【丢失路由跳转404页面】问题
      （因为异步获取路由优先级比静态路由表低，导致404路由在异步添加的路由之前）
  */
  getInfo({ commit, state }) {
    return new Promise((resolve, reject) => {
      user
        .getInfo(state.token)
        .then((response) => {
          const { data } = response

          if (!data) {
            reject('Verification failed, please Login again.')
          }

          const { name, avatar } = data

          commit('SET_NAME', name)
          commit('SET_AVATAR', avatar)
          resolve(data)
        })
        .catch((error) => {
          reject(error)
        })
    })
  },

  // user logout
  logout({ commit, state }) {
    const id = JSON.parse(getUserInfo()).id
    return new Promise((resolve, reject) => {
      // 未有退出登录接口
      user
        .logClear(id)
        .then(() => {
          removeToken() // must remove  token  first
          removeUserInfo()
          resetRouter()
          commit('RESET_STATE')
          resolve()
        })
        .catch((error) => {
          reject(error)
        })
    })
  },

  // remove token
  resetToken({ commit }) {
    return new Promise((resolve) => {
      removeToken() // must remove  token  first
      removeUserInfo()
      commit('RESET_STATE')
      resolve()
    })
  },
}

export default {
  namespaced: true,
  state,
  mutations,
  actions,
}
