import axios from 'axios'
import { MessageBox, Message } from 'element-ui'
import store from '@/store'
import { getToken } from '@/utils/auth'

// 用于中止web请求
const abortController = new AbortController()

const service = axios.create({
  baseURL: import.meta.env.VITE_APP_BASE_API, // url = base url + request url
  // baseURL: '/api', // url = base url + request url
  // withCredentials: true, // send cookies when cross-domain requests
  // timeout: 5000,
})

service.interceptors.request.use(
  (config) => {
    if (store.getters.token) {
      config.headers.Authorization = 'Bearer ' + getToken()
    }
    return config
  },
  (error) => {
    return Promise.reject(error)
  }
)

service.interceptors.response.use(
  (response) => {
    const { data } = response
    // 状态码200，code=1 成功
    if (data.code != 1) {
      if (data instanceof Blob) {
        // 文件流
        return data
      } else {
        Message({
          message: data.msg,
          type: 'error',
          duration: 3 * 1000,
        })
      }

      return Promise.reject(new Error(data || 'Error'))
    } else {
      return data
    }
  },
  (error) => {
    console.log('error: ', error.response)
    const { data, status } = error.response
    if (typeof data.msg === 'undefined') {
      Message({
        message: 'Error:请联系管理员',
        type: 'error',
        duration: 5 * 1000,
      })
    }

    if ([303, 304, 305].includes(status)) {
      abortController.abort() // 中止正在进行的请求
      MessageBox.confirm('登录失效，请重新登录1！', {
        confirmButtonText: '确定',
        showCancelButton: false,
        showClose: false,
        closeOnClickModal: false,
        type: 'warning',
      }).then(() => {
        store.dispatch('user/resetToken').then(() => {
          location.href = './'
        })
      })
    }
    if (data.msg != 'token失效') {
      Message({
        message: data.msg,
        type: 'error',
        duration: 3 * 1000,
      })
    }
    if (axios.isCancel(error)) {
      // 取消请求的情况下，终端Promise调用链
      return new Promise(() => {})
    } else {
      return Promise.reject(error)
    }
  }
)

export default service
